@@ -16,10 +16,6 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "xfce-mime-helper-chooser.h"
 #include "xfce-mime-helper-enum-types.h"
 #include "xfce-mime-helper-utils.h"
@@ -350,6 +346,10 @@ menu_activate (GtkWidget *item,
     N_ ("Failed to set default Mail Reader"),
     N_ ("Failed to set default File Manager"),
     N_ ("Failed to set default Terminal Emulator"),
+    N_ ("Failed to set default Text Editor"),
+    N_ ("Failed to set default Image Viewer"),
+    N_ ("Failed to set default Audio Player"),
+    N_ ("Failed to set default Video Player"),
   };
 
   XfceMimeHelper *helper;
@@ -534,6 +534,10 @@ menu_activate_other (GtkWidget *item,
     N_ ("Choose a custom Mail Reader"),
     N_ ("Choose a custom File Manager"),
     N_ ("Choose a custom Terminal Emulator"),
+    N_ ("Choose a custom Text Editor"),
+    N_ ("Choose a custom Image Viewer"),
+    N_ ("Choose a custom Audio Player"),
+    N_ ("Choose a custom Video Player"),
   };
 
   static const gchar *BROWSE_MESSAGES[] = {
@@ -541,6 +545,10 @@ menu_activate_other (GtkWidget *item,
     N_ ("Specify the application you want to use\nas default Mail Reader for Xfce:"),
     N_ ("Specify the application you want to use\nas default File Manager for Xfce:"),
     N_ ("Specify the application you want to use\nas default Terminal Emulator for Xfce:"),
+    N_ ("Specify the application you want to use\nas default Text Editor for Xfce:"),
+    N_ ("Specify the application you want to use\nas default Image Viewer for Xfce:"),
+    N_ ("Specify the application you want to use\nas default Audio Player for Xfce:"),
+    N_ ("Specify the application you want to use\nas default Video Player for Xfce:"),
   };
 
   const gchar *command;
