@@ -17,9 +17,6 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #ifdef HAVE_XFCE_REVISION_H
 #include "xfce-revision.h"
 #endif
@@ -65,8 +62,8 @@ save_window_size (GtkWidget *dialog,
     {
         /* save window size */
         gtk_window_get_size (GTK_WINDOW (dialog), &width, &height);
-        xfconf_channel_set_int (channel, "/last/window-width", width),
-            xfconf_channel_set_int (channel, "/last/window-height", height);
+        xfconf_channel_set_int (channel, "/last/window-width", width);
+        xfconf_channel_set_int (channel, "/last/window-height", height);
         xfconf_channel_set_int (channel, "/last/paned-position", paned_pos);
     }
 }
