@@ -17,26 +17,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#include <gtk/gtk.h>
-#include <dbus/dbus-glib.h>
-
-#include <libxfce4util/libxfce4util.h>
-#include <libxfce4ui/libxfce4ui.h>
-#include <xfconf/xfconf.h>
-
 #include "xfce-settings-prop-dialog.h"
 
+#include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4util/libxfce4util.h>
+
 
 
 struct _XfceSettingsPropDialogClass
@@ -50,34 +35,39 @@ struct _XfceSettingsPropDialog
 
     XfconfChannel *channel;
 
-    GValue         prop_value;
+    GValue prop_value;
 
-    GtkWidget     *prop_name;
-    GtkWidget     *prop_type;
-    GtkWidget     *prop_string;
-    GtkWidget     *prop_integer;
-    GtkWidget     *prop_bool;
+    GtkWidget *prop_name;
+    GtkWidget *prop_type;
+    GtkWidget *prop_string;
+    GtkWidget *prop_integer;
+    GtkWidget *prop_bool;
 };
 
 typedef struct
 {
     const gchar *name;
-    GType        type;
-}
-ValueTypes;
+    GType type;
+} ValueTypes;
 
 
 
-static void     xfce_settings_prop_dialog_finalize             (GObject                   *object);
-static void     xfce_settings_prop_dialog_response             (GtkDialog                 *widget,
-                                                                gint                       response_id);
-static void     xfce_settings_prop_dialog_visible_bind         (GtkWidget                 *widget,
-                                                                GtkWidget                 *label);
-static void     xfce_settings_prop_dialog_entry_validate       (GtkWidget                 *entry,
-                                                                XfceSettingsPropDialog    *dialog);
-static void     xfce_settings_prop_dialog_button_toggled       (GtkWidget                 *button);
-static void     xfce_settings_prop_dialog_type_changed         (GtkWidget                 *combo,
-                                                                XfceSettingsPropDialog    *dialog);
+static void
+xfce_settings_prop_dialog_finalize (GObject *object);
+static void
+xfce_settings_prop_dialog_response (GtkDialog *widget,
+                                    gint response_id);
+static void
+xfce_settings_prop_dialog_visible_bind (GtkWidget *widget,
+                                        GtkWidget *label);
+static void
+xfce_settings_prop_dialog_entry_validate (GtkWidget *entry,
+                                          XfceSettingsPropDialog *dialog);
+static void
+xfce_settings_prop_dialog_button_toggled (GtkWidget *button);
+static void
+xfce_settings_prop_dialog_type_changed (GtkWidget *combo,
+                                        XfceSettingsPropDialog *dialog);
 
 
 
@@ -85,25 +75,24 @@ G_DEFINE_TYPE (XfceSettingsPropDialog, xfce_settings_prop_dialog, GTK_TYPE_DIALO
 
 
 
-static ValueTypes value_types[] =
-{
-  { N_("Empty"), G_TYPE_NONE },
-  { N_("String"), G_TYPE_STRING },
-  { N_("Boolean"), G_TYPE_BOOLEAN },
-  { N_("Int"), G_TYPE_INT },
-  { N_("Double"), G_TYPE_DOUBLE },
-  { N_("Unsigned Int"), G_TYPE_UINT },
-  { N_("Int64"), G_TYPE_INT64 },
-  { N_("Unsigned Int64"), G_TYPE_UINT64 }
+static ValueTypes value_types[] = {
+    { N_ ("Empty"), G_TYPE_NONE },
+    { N_ ("String"), G_TYPE_STRING },
+    { N_ ("Boolean"), G_TYPE_BOOLEAN },
+    { N_ ("Int"), G_TYPE_INT },
+    { N_ ("Double"), G_TYPE_DOUBLE },
+    { N_ ("Unsigned Int"), G_TYPE_UINT },
+    { N_ ("Int64"), G_TYPE_INT64 },
+    { N_ ("Unsigned Int64"), G_TYPE_UINT64 }
 };
 
 
 
 enum
 {
-  COLUMN_NAME,
-  COLUMN_ID,
-  N_COLUMNS
+    COLUMN_NAME,
+    COLUMN_ID,
+    N_COLUMNS
 };
 
 
@@ -111,7 +100,7 @@ enum
 static void
 xfce_settings_prop_dialog_class_init (XfceSettingsPropDialogClass *klass)
 {
-    GObjectClass   *gobject_class;
+    GObjectClass *gobject_class;
     GtkDialogClass *gtkdialog_class;
 
     gobject_class = G_OBJECT_CLASS (klass);
@@ -126,54 +115,55 @@ xfce_settings_prop_dialog_class_init (XfceSettingsPropDialogClass *klass)
 static void
 xfce_settings_prop_dialog_init (XfceSettingsPropDialog *dialog)
 {
-    GtkWidget       *table;
-    GtkWidget       *content_area;
-    GtkWidget       *label;
-    GtkWidget       *entry;
-    GtkWidget       *combo;
-    GtkWidget       *spin;
-    GtkWidget       *toggle;
-    GtkListStore    *store;
-    guint            i;
+    GtkWidget *table;
+    GtkWidget *content_area;
+    GtkWidget *label;
+    GtkWidget *entry;
+    GtkWidget *combo;
+    GtkWidget *spin;
+    GtkWidget *toggle;
+    GtkListStore *store;
+    guint i;
     GtkCellRenderer *render;
-    GtkWidget       *save_button;
+    GtkWidget *save_button;
 
+    gtk_window_set_icon_name (GTK_WINDOW (dialog), "list-add");
     gtk_window_set_title (GTK_WINDOW (dialog), _("New Property"));
     gtk_window_set_default_size (GTK_WINDOW (dialog), 300, 200);
-    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_SAVE, GTK_RESPONSE_OK, NULL);
+    gtk_dialog_add_buttons (
+        GTK_DIALOG (dialog), _("Cancel"), GTK_RESPONSE_CANCEL, _("Save"), GTK_RESPONSE_OK, NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
     save_button = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
     gtk_widget_set_sensitive (save_button, FALSE);
 
-    table = gtk_table_new (5, 2, FALSE);
+    table = gtk_grid_new ();
+    gtk_container_set_border_width (GTK_CONTAINER (table), 6);
     content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
     gtk_box_pack_start (GTK_BOX (content_area), table, TRUE, TRUE, 0);
-    gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+    gtk_grid_set_column_spacing (GTK_GRID (table), 12);
+    gtk_grid_set_row_spacing (GTK_GRID (table), 6);
     gtk_container_set_border_width (GTK_CONTAINER (table), 6);
     gtk_widget_show (table);
 
     label = gtk_label_new_with_mnemonic (_("_Property:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.50);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
-                      GTK_FILL, GTK_FILL, 0, 0);
-    gtk_table_set_row_spacing (GTK_TABLE (table), 0, 6);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+    gtk_grid_attach (GTK_GRID (table), label, 0, 0, 1, 1);
     gtk_widget_show (label);
 
     dialog->prop_name = entry = gtk_entry_new ();
-    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+    gtk_widget_set_hexpand (GTK_WIDGET (entry), TRUE);
+    gtk_grid_attach (GTK_GRID (table), entry, 1, 0, 1, 1);
     xfce_settings_prop_dialog_visible_bind (entry, label);
     g_signal_connect (G_OBJECT (entry), "changed",
-        G_CALLBACK (xfce_settings_prop_dialog_entry_validate), dialog);
+                      G_CALLBACK (xfce_settings_prop_dialog_entry_validate), dialog);
     gtk_widget_show (entry);
 
     label = gtk_label_new_with_mnemonic (_("_Type:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.50);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
-                      GTK_FILL, GTK_FILL, 0, 0);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+    gtk_grid_attach (GTK_GRID (table), label, 0, 1, 1, 1);
     gtk_widget_show (label);
 
     store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_UINT);
@@ -185,13 +175,11 @@ xfce_settings_prop_dialog_init (XfceSettingsPropDialog *dialog)
     }
 
     dialog->prop_type = combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
-    gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 1, 2,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
-    gtk_table_set_row_spacing (GTK_TABLE (table), 1, 6);
+    gtk_grid_attach (GTK_GRID (table), combo, 1, 1, 1, 1);
     xfce_settings_prop_dialog_visible_bind (combo, label);
     gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
     g_signal_connect (G_OBJECT (combo), "changed",
-        G_CALLBACK (xfce_settings_prop_dialog_type_changed), dialog);
+                      G_CALLBACK (xfce_settings_prop_dialog_type_changed), dialog);
     gtk_widget_show (combo);
     g_object_unref (G_OBJECT (store));
 
@@ -202,38 +190,35 @@ xfce_settings_prop_dialog_init (XfceSettingsPropDialog *dialog)
 
     /* strings */
     label = gtk_label_new_with_mnemonic (_("_Value:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.50);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
-                      GTK_FILL, GTK_FILL, 0, 0);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+    gtk_grid_attach (GTK_GRID (table), label, 0, 2, 1, 1);
 
     entry = dialog->prop_string = gtk_entry_new ();
-    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+    gtk_grid_attach (GTK_GRID (table), entry, 1, 2, 1, 1);
     xfce_settings_prop_dialog_visible_bind (entry, label);
 
     /* integers */
     label = gtk_label_new_with_mnemonic (_("_Value:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.50);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
-                      GTK_FILL, GTK_FILL, 0, 0);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+    gtk_grid_attach (GTK_GRID (table), label, 0, 3, 1, 1);
 
     spin = dialog->prop_integer = gtk_spin_button_new_with_range (0.00, 0.00, 1.00);
     gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE);
-    gtk_table_attach (GTK_TABLE (table), spin, 1, 2, 3, 4,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+    gtk_grid_attach (GTK_GRID (table), spin, 1, 3, 1, 1);
     xfce_settings_prop_dialog_visible_bind (spin, label);
 
     /* bool */
     label = gtk_label_new_with_mnemonic (_("_Value:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.50);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
-                      GTK_FILL, GTK_FILL, 0, 0);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+    gtk_grid_attach (GTK_GRID (table), label, 0, 4, 1, 1);
 
     toggle = dialog->prop_bool = gtk_toggle_button_new_with_label ("FALSE");
     g_signal_connect (G_OBJECT (toggle), "toggled",
-        G_CALLBACK (xfce_settings_prop_dialog_button_toggled), NULL);
-    gtk_table_attach (GTK_TABLE (table), toggle, 1, 2, 4, 5,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+                      G_CALLBACK (xfce_settings_prop_dialog_button_toggled), NULL);
+    gtk_grid_attach (GTK_GRID (table), toggle, 1, 4, 1, 1);
     xfce_settings_prop_dialog_visible_bind (toggle, label);
 }
 
@@ -257,14 +242,14 @@ xfce_settings_prop_dialog_finalize (GObject *object)
 
 static void
 xfce_settings_prop_dialog_response (GtkDialog *widget,
-                                    gint       response_id)
+                                    gint response_id)
 {
     XfceSettingsPropDialog *dialog = XFCE_SETTINGS_PROP_DIALOG (widget);
-    const gchar            *property;
-    ValueTypes             *value_type;
-    GValue                  value = { 0, };
-    gdouble                 spin_value;
-    gint                    active;
+    const gchar *property;
+    ValueTypes *value_type;
+    GValue value = G_VALUE_INIT;
+    gdouble spin_value;
+    gint active;
 
     g_return_if_fail (XFCONF_IS_CHANNEL (dialog->channel));
 
@@ -285,14 +270,12 @@ xfce_settings_prop_dialog_response (GtkDialog *widget,
 
             case G_TYPE_STRING:
                 g_value_init (&value, G_TYPE_STRING);
-                g_value_set_static_string (&value,
-                    gtk_entry_get_text (GTK_ENTRY (dialog->prop_string)));
+                g_value_set_static_string (&value, gtk_entry_get_text (GTK_ENTRY (dialog->prop_string)));
                 break;
 
             case G_TYPE_BOOLEAN:
                 g_value_init (&value, G_TYPE_BOOLEAN);
-                g_value_set_boolean (&value,
-                    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->prop_bool)));
+                g_value_set_boolean (&value, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->prop_bool)));
                 break;
 
             case G_TYPE_INT:
@@ -332,9 +315,9 @@ xfce_settings_prop_dialog_response (GtkDialog *widget,
 
 
 static void
-xfce_settings_prop_dialog_visible_changed (GtkWidget  *widget,
+xfce_settings_prop_dialog_visible_changed (GtkWidget *widget,
                                            GParamSpec *pspec,
-                                           GtkWidget  *label)
+                                           GtkWidget *label)
 {
     g_return_if_fail (GTK_IS_WIDGET (widget));
     g_return_if_fail (GTK_IS_LABEL (label));
@@ -345,9 +328,9 @@ xfce_settings_prop_dialog_visible_changed (GtkWidget  *widget,
 
 
 static void
-xfce_settings_prop_dialog_sensitive_changed (GtkWidget  *widget,
+xfce_settings_prop_dialog_sensitive_changed (GtkWidget *widget,
                                              GParamSpec *pspec,
-                                             GtkWidget  *label)
+                                             GtkWidget *label)
 {
     g_return_if_fail (GTK_IS_WIDGET (widget));
     g_return_if_fail (GTK_IS_LABEL (label));
@@ -363,17 +346,17 @@ xfce_settings_prop_dialog_visible_bind (GtkWidget *widget,
 {
     gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
     g_signal_connect (G_OBJECT (widget), "notify::visible",
-        G_CALLBACK (xfce_settings_prop_dialog_visible_changed), label);
+                      G_CALLBACK (xfce_settings_prop_dialog_visible_changed), label);
     g_signal_connect (G_OBJECT (widget), "notify::sensitive",
-        G_CALLBACK (xfce_settings_prop_dialog_sensitive_changed), label);
+                      G_CALLBACK (xfce_settings_prop_dialog_sensitive_changed), label);
 }
 
 
 
 /* Copied from xfconfd/xfconf-backend.c */
 static gboolean
-xfconf_property_is_valid (const gchar  *property,
-                          GError      **error)
+xfconf_property_is_valid (const gchar *property,
+                          GError **error)
 {
     const gchar *p = property;
 
@@ -402,16 +385,18 @@ xfconf_property_is_valid (const gchar  *property,
     {
         if (!(*p >= 'A' && *p <= 'Z') && !(*p >= 'a' && *p <= 'z')
             && !(*p >= '0' && *p <= '9')
-            && *p != '_' && *p != '-' && *p != '/'
-            && !(*p == '<' || *p == '>'))
+            && *p != '_' && *p != '-' && *p != '/' && *p != '{' && *p != '}'
+            && !(*p == '<' || *p == '>') && *p != '|' && *p != ','
+            && *p != '[' && *p != ']' && *p != '.' && *p != ':')
         {
             if (error != NULL)
             {
                 g_set_error (error, XFCONF_ERROR,
                              XFCONF_ERROR_INVALID_PROPERTY,
                              _("Property names can only include the ASCII "
-                               "characters A-Z, a-z, 0-9, '_', '-', '<' "
-                               "and '>', as well as '/' as a separator"));
+                               "characters A-Z, a-z, 0-9, '_', '-', ':', '.', "
+                               "',', '[', ']', '{', '}', '<' and '>', as well "
+                               "as '/' as a separator"));
             }
             return FALSE;
         }
@@ -448,13 +433,13 @@ xfconf_property_is_valid (const gchar  *property,
 
 
 static void
-xfce_settings_prop_dialog_entry_validate (GtkWidget              *entry,
+xfce_settings_prop_dialog_entry_validate (GtkWidget *entry,
                                           XfceSettingsPropDialog *dialog)
 {
-    GtkWidget   *save_button;
+    GtkWidget *save_button;
     const gchar *text;
-    gboolean     is_valid = FALSE;
-    GError      *error = NULL;
+    gboolean is_valid = FALSE;
+    GError *error = NULL;
 
     text = gtk_entry_get_text (GTK_ENTRY (entry));
 
@@ -462,8 +447,8 @@ xfce_settings_prop_dialog_entry_validate (GtkWidget              *entry,
     {
         is_valid = xfconf_property_is_valid (text, &error);
 
-        gtk_entry_set_icon_from_stock (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY,
-                                       is_valid ? NULL : GTK_STOCK_DIALOG_ERROR);
+        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY,
+                                           is_valid ? NULL : "dialog-error");
         gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY,
                                          is_valid ? NULL : error->message);
 
@@ -472,7 +457,7 @@ xfce_settings_prop_dialog_entry_validate (GtkWidget              *entry,
     }
     else
     {
-        gtk_entry_set_icon_from_stock (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, NULL);
+        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, NULL);
     }
 
     save_button = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -493,11 +478,11 @@ xfce_settings_prop_dialog_button_toggled (GtkWidget *button)
 
 
 static void
-xfce_settings_prop_dialog_type_changed (GtkWidget              *combo,
+xfce_settings_prop_dialog_type_changed (GtkWidget *combo,
                                         XfceSettingsPropDialog *dialog)
 {
-    gint          active;
-    ValueTypes   *value_type;
+    gint active;
+    ValueTypes *value_type;
     const GValue *value = &dialog->prop_value;
 
     gtk_widget_hide (dialog->prop_string);
@@ -587,7 +572,7 @@ xfce_settings_prop_dialog_type_changed (GtkWidget              *combo,
             gtk_widget_show (dialog->prop_integer);
             gtk_widget_grab_focus (dialog->prop_integer);
             gtk_spin_button_set_range (GTK_SPIN_BUTTON (dialog->prop_integer),
-                                       G_MININT64, G_MAXINT64);
+                                       G_MININT64, (gdouble) G_MAXINT64);
 
             if (G_VALUE_HOLDS_INT64 (value))
             {
@@ -600,7 +585,7 @@ xfce_settings_prop_dialog_type_changed (GtkWidget              *combo,
             gtk_widget_show (dialog->prop_integer);
             gtk_widget_grab_focus (dialog->prop_integer);
             gtk_spin_button_set_range (GTK_SPIN_BUTTON (dialog->prop_integer),
-                                       0, G_MAXUINT64);
+                                       0, (gdouble) G_MAXUINT64);
 
             if (G_VALUE_HOLDS_UINT64 (value))
             {
@@ -615,7 +600,7 @@ xfce_settings_prop_dialog_type_changed (GtkWidget              *combo,
 
 static void
 xfce_settings_prop_dialog_type_set_active (XfceSettingsPropDialog *dialog,
-                                           GType                   value_type)
+                                           GType value_type)
 {
     guint i;
 
@@ -632,9 +617,9 @@ xfce_settings_prop_dialog_type_set_active (XfceSettingsPropDialog *dialog,
 
 
 GtkWidget *
-xfce_settings_prop_dialog_new (GtkWindow     *parent,
+xfce_settings_prop_dialog_new (GtkWindow *parent,
                                XfconfChannel *channel,
-                               const gchar   *property)
+                               const gchar *property)
 {
     XfceSettingsPropDialog *dialog;
 
@@ -643,18 +628,18 @@ xfce_settings_prop_dialog_new (GtkWindow     *parent,
 
     dialog = g_object_new (XFCE_TYPE_SETTINGS_PROP_DIALOG, NULL);
 
-    dialog->channel = g_object_ref (G_OBJECT (channel));
+    dialog->channel = (XfconfChannel *) g_object_ref (G_OBJECT (channel));
 
     if (property != NULL)
     {
         gtk_entry_set_text (GTK_ENTRY (dialog->prop_name), property);
         gtk_editable_set_editable (GTK_EDITABLE (dialog->prop_name), FALSE);
         gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Property"));
+        gtk_window_set_icon_name (GTK_WINDOW (dialog), "gtk-edit");
 
         if (xfconf_channel_get_property (channel, property, &dialog->prop_value))
         {
-            xfce_settings_prop_dialog_type_set_active (dialog,
-                G_VALUE_TYPE (&dialog->prop_value));
+            xfce_settings_prop_dialog_type_set_active (dialog, G_VALUE_TYPE (&dialog->prop_value));
             gtk_widget_set_sensitive (dialog->prop_type, FALSE);
         }
     }
@@ -674,11 +659,11 @@ xfce_settings_prop_dialog_new (GtkWindow     *parent,
 
 void
 xfce_settings_prop_dialog_set_parent_property (XfceSettingsPropDialog *dialog,
-                                               const gchar            *property)
+                                               const gchar *property)
 {
     gchar *p;
-    gint   length = -1;
-    gint   pos = 0;
+    gint length = -1;
+    gint pos = 0;
 
     g_return_if_fail (XFCE_IS_SETTINGS_PROP_DIALOG (dialog));
 
@@ -686,7 +671,7 @@ xfce_settings_prop_dialog_set_parent_property (XfceSettingsPropDialog *dialog,
     {
         p = strrchr (property, '/');
         if (G_LIKELY (p != NULL))
-          length = (p - property) + 1;
+            length = (p - property) + 1;
 
         gtk_editable_insert_text (GTK_EDITABLE (dialog->prop_name),
                                   property, length, &pos);
