@@ -16,53 +16,57 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#include <glib.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkx.h>
-#include <xfconf/xfconf.h>
-#include <libxfce4util/libxfce4util.h>
-
-#include <dbus/dbus-glib.h>
-
-#include "debug.h"
-#include "pointers.h"
 #include "pointers-defines.h"
+#include "pointers.h"
 
+#include "common/debug.h"
+
+#include <gdk/gdkx.h>
+#include <libxfce4util/libxfce4util.h>
+#include <xfconf/xfconf.h>
+
+#ifdef HAVE_LIBINPUT
+#include <libinput-properties.h>
+#endif /* HAVE_LIBINPUT */
+
+#include <locale.h>
 
 #define MAX_DENOMINATOR (100.00)
 
-#define XFCONF_TYPE_G_VALUE_ARRAY (dbus_g_type_get_collection ("GPtrArray", G_TYPE_VALUE))
+#ifdef XI_PROP_ENABLED
+#define DEVICE_ENABLED XI_PROP_ENABLED
+#else
+#define DEVICE_ENABLED "Device Enabled"
+#endif /* XI_PROP_ENABLED */
 
-
-
-static void             xfce_pointers_helper_finalize                 (GObject            *object);
-static void             xfce_pointers_helper_syndaemon_stop           (XfcePointersHelper *helper);
-static void             xfce_pointers_helper_syndaemon_check          (XfcePointersHelper *helper);
-static void             xfce_pointers_helper_restore_devices          (XfcePointersHelper *helper,
-                                                                       XID                *xid);
-static void             xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
-                                                                       const gchar        *property_name,
-                                                                       const GValue       *value,
-                                                                       XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_finalize (GObject *object);
+static void
+xfce_pointers_helper_syndaemon_stop (XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
+                                      XID *xid);
+static void
+xfce_pointers_helper_channel_property_changed (XfconfChannel *channel,
+                                               const gchar *property_name,
+                                               const GValue *value,
+                                               XfcePointersHelper *helper);
 #ifdef DEVICE_HOTPLUGGING
-static GdkFilterReturn  xfce_pointers_helper_event_filter             (GdkXEvent          *xevent,
-                                                                       GdkEvent           *gdk_event,
-                                                                       gpointer            user_data);
+static GdkFilterReturn
+xfce_pointers_helper_event_filter (GdkXEvent *xevent,
+                                   GdkEvent *gdk_event,
+                                   gpointer user_data);
 #endif
+#if defined(DEVICE_PROPERTIES) || defined(HAVE_LIBINPUT)
+static void
+xfce_pointers_helper_change_property (XDeviceInfo *device_info,
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      const gchar *prop_name,
+                                      const GValue *value);
+#endif /* DEVICE_PROPERTIES || HAVE_LIBINPUT */
 
 
 
@@ -73,29 +77,28 @@ struct _XfcePointersHelperClass
 
 struct _XfcePointersHelper
 {
-    GObject  __parent__;
+    GObject __parent__;
 
     /* xfconf channel */
     XfconfChannel *channel;
 
 #ifdef DEVICE_PROPERTIES
-    GPid           syndaemon_pid;
+    GPid syndaemon_pid;
 #endif
 
 #ifdef DEVICE_HOTPLUGGING
     /* device presence event type */
-    gint           device_presence_event_type;
+    gint device_presence_event_type;
 #endif
 };
 
 typedef struct
 {
-    Display     *xdisplay;
-    XDevice     *device;
+    Display *xdisplay;
+    XDevice *device;
     XDeviceInfo *device_info;
-    gsize        prop_name_len;
-}
-XfcePointerData;
+    gsize prop_name_len;
+} XfcePointerData;
 
 
 
@@ -117,9 +120,9 @@ static void
 xfce_pointers_helper_init (XfcePointersHelper *helper)
 {
     XExtensionVersion *version = NULL;
-    Display           *xdisplay;
+    Display *xdisplay;
 #ifdef DEVICE_HOTPLUGGING
-    XEventClass        event_class;
+    XEventClass event_class;
 #endif
 
     /* get the default display */
@@ -129,7 +132,8 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
     version = XGetExtensionVersion (xdisplay, INAME);
 
     /* check for Xi */
-    if (version == NULL || !version->present)
+    if (version == NULL || ((long) version) == NoSuchExtension
+        || !version->present)
     {
         g_critical ("XI is not present.");
     }
@@ -154,7 +158,7 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
 
         /* monitor the channel */
         g_signal_connect (G_OBJECT (helper->channel), "property-changed",
-             G_CALLBACK (xfce_pointers_helper_channel_property_changed), helper);
+                          G_CALLBACK (xfce_pointers_helper_channel_property_changed), helper);
 
         /* launch syndaemon if required */
         xfce_pointers_helper_syndaemon_check (helper);
@@ -163,18 +167,21 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
         if (G_LIKELY (xdisplay != NULL))
         {
             /* monitor device changes */
-            gdk_error_trap_push ();
+            gdk_x11_display_error_trap_push (gdk_display_get_default ());
             DevicePresence (xdisplay, helper->device_presence_event_type, event_class);
             XSelectExtensionEvent (xdisplay, RootWindow (xdisplay, DefaultScreen (xdisplay)), &event_class, 1);
 
             /* add an event filter */
-            if (gdk_error_trap_pop () == 0)
+            if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) == 0)
                 gdk_window_add_filter (NULL, xfce_pointers_helper_event_filter, helper);
             else
                 g_warning ("Failed to create device filter");
         }
 #endif
     }
+
+    if (version)
+        XFree (version);
 }
 
 
@@ -189,6 +196,74 @@ xfce_pointers_helper_finalize (GObject *object)
 
 
 
+#ifdef HAVE_LIBINPUT
+static gboolean
+xfce_pointers_is_enabled (Display *xdisplay,
+                          XDevice *device)
+{
+    Atom prop, type;
+    gulong n_items, bytes_after;
+    gint rc, format;
+    guchar *data;
+    gboolean enabled;
+
+    prop = XInternAtom (xdisplay, DEVICE_ENABLED, False);
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
+    rc = XGetDeviceProperty (xdisplay, device, prop, 0, 1, False,
+                             XA_INTEGER, &type, &format, &n_items,
+                             &bytes_after, &data);
+    gdk_x11_display_error_trap_pop_ignored (gdk_display_get_default ());
+    if (rc == Success)
+    {
+        enabled = (gboolean) *data;
+        XFree (data);
+        return (enabled);
+    }
+
+    return FALSE;
+}
+
+
+
+static gboolean
+xfce_pointers_libinput_prop_available (Display *xdisplay,
+                                       XDevice *device,
+                                       const char *propname)
+{
+    Atom prop, type;
+    gulong n_items, bytes_after;
+    gint rc, format;
+    guchar *data;
+
+    prop = XInternAtom (xdisplay, propname, False);
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
+    rc = XGetDeviceProperty (xdisplay, device, prop, 0, 1, False,
+                             XA_INTEGER, &type, &format, &n_items,
+                             &bytes_after, &data);
+    gdk_x11_display_error_trap_pop_ignored (gdk_display_get_default ());
+    if (rc == Success)
+    {
+        XFree (data);
+        return (n_items > 0);
+    }
+
+    return FALSE;
+}
+
+
+
+static gboolean
+xfce_pointers_is_libinput (Display *xdisplay,
+                           XDevice *device)
+{
+    /* check both properties because not all devices have LIBINPUT_PROP_LEFT_HANDED */
+    return xfce_pointers_libinput_prop_available (xdisplay, device, LIBINPUT_PROP_LEFT_HANDED)
+           || xfce_pointers_libinput_prop_available (xdisplay, device, LIBINPUT_PROP_NATURAL_SCROLL);
+}
+#endif /* HAVE_LIBINPUT */
+
+
+
 static void
 xfce_pointers_helper_syndaemon_stop (XfcePointersHelper *helper)
 {
@@ -211,25 +286,27 @@ static void
 xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 {
 #ifdef DEVICE_PROPERTIES
-    Display     *xdisplay = GDK_DISPLAY ();
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
     XDeviceInfo *device_list;
-    XDevice     *device;
-    gint         n, ndevices;
-    Atom         touchpad_type;
-    Atom         touchpad_off_prop;
-    Atom        *props;
-    gint         i, nprops;
-    gboolean     have_synaptics = FALSE;
-    gchar       *args[] = { "syndaemon", "-i", "2.0", "-K", "-R", NULL };
-    GError      *error = NULL;
+    XDevice *device;
+    gint n, ndevices;
+    Atom touchpad_type;
+    Atom touchpad_off_prop;
+    Atom *props;
+    gint i, nprops;
+    gboolean have_synaptics = FALSE;
+    gdouble disable_duration;
+    gchar disable_duration_string[64];
+    gchar *args[] = { "syndaemon", "-i", disable_duration_string, "-K", "-R", NULL };
+    GError *error = NULL;
 
     /* only stop a running daemon */
     if (!xfconf_channel_get_bool (helper->channel, "/DisableTouchpadWhileTyping", FALSE))
         goto start_stop_daemon;
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     device_list = XListInputDevices (xdisplay, &ndevices);
-    if (gdk_error_trap_pop () != 0 || device_list == NULL)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device_list == NULL)
         goto start_stop_daemon;
 
     touchpad_type = XInternAtom (xdisplay, XI_TOUCHPAD, True);
@@ -241,18 +318,18 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
         if (device_list[n].type != touchpad_type)
             continue;
 
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         device = XOpenDevice (xdisplay, device_list[n].id);
-        if (gdk_error_trap_pop () != 0 || device == NULL)
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device == NULL)
         {
             g_critical ("Unable to open device %s", device_list[n].name);
             break;
         }
 
         /* look for the Synaptics Off property */
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         props = XListDeviceProperties (xdisplay, device, &nprops);
-        if (gdk_error_trap_pop () == 0
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) == 0
             && props != NULL)
         {
             for (i = 0; !have_synaptics && i < nprops; i++)
@@ -269,27 +346,29 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 
     XFreeDeviceList (device_list);
 
-    start_stop_daemon:
+start_stop_daemon:
+
+    /* stop the daemon in any case */
+    xfce_pointers_helper_syndaemon_stop (helper);
 
     if (have_synaptics)
     {
-        if (helper->syndaemon_pid == 0)
-        {
-            if (!g_spawn_async (NULL, args, NULL, G_SPAWN_SEARCH_PATH,
-                                NULL, NULL, &helper->syndaemon_pid, &error))
-            {
-                g_critical ("Spawning syndaemon failed: %s", error->message);
-                g_error_free (error);
-            }
+        disable_duration = xfconf_channel_get_double (helper->channel,
+                                                      "/DisableTouchpadDuration",
+                                                      2.0);
+        setlocale (LC_NUMERIC, "C"); /* syndaemon needs a dot for the float. Nothing localized! */
+        g_snprintf (disable_duration_string, sizeof (disable_duration_string),
+                    "%.1f", disable_duration);
 
-            xfsettings_dbg (XFSD_DEBUG_POINTERS, "Started syndaemon with pid %d",
-                            helper->syndaemon_pid);
+        if (!g_spawn_async (NULL, args, NULL, G_SPAWN_SEARCH_PATH,
+                            NULL, NULL, &helper->syndaemon_pid, &error))
+        {
+            g_critical ("Spawning syndaemon failed: %s", error->message);
+            g_error_free (error);
         }
-    }
-    else
-    {
-        /* stop the daemon */
-        xfce_pointers_helper_syndaemon_stop (helper);
+
+        xfsettings_dbg (XFSD_DEBUG_POINTERS, "Started syndaemon with pid %d",
+                        helper->syndaemon_pid);
     }
 #endif
 }
@@ -297,15 +376,15 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 
 
 static gboolean
-xfce_pointers_helper_change_button_mapping_swap (guchar   *buttonmap,
-                                                 gshort    num_buttons,
-                                                 gint      id_1,
-                                                 gint      id_2,
-                                                 gboolean  reverse)
+xfce_pointers_helper_change_button_mapping_swap (guchar *buttonmap,
+                                                 gshort num_buttons,
+                                                 gint id_1,
+                                                 gint id_2,
+                                                 gboolean reverse)
 {
     gshort n;
-    gint   id_a = -1;
-    gint   id_b = -1;
+    gint id_a = -1;
+    gint id_b = -1;
 
     /* figure out the position of the id_1 and id_2 buttons in the map */
     for (n = 0; n < num_buttons; n++)
@@ -338,18 +417,47 @@ xfce_pointers_helper_change_button_mapping_swap (guchar   *buttonmap,
 
 static void
 xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
-                                            XDevice     *device,
-                                            Display     *xdisplay,
-                                            gint         right_handed,
-                                            gint         reverse_scrolling)
+                                            XDevice *device,
+                                            Display *xdisplay,
+                                            gint right_handed,
+                                            gint reverse_scrolling)
 {
-    XAnyClassPtr  ptr;
-    gshort        num_buttons = 0;
-    guchar       *buttonmap;
-    gboolean      map_changed = FALSE;
-    gint          n;
-    gint          right_button;
-    GString      *readable_map;
+    XAnyClassPtr ptr;
+    gshort num_buttons = 0;
+    guchar *buttonmap;
+    gboolean map_changed = FALSE;
+    gint n;
+    gint right_button;
+    GString *readable_map;
+
+#ifdef HAVE_LIBINPUT
+    if (xfce_pointers_is_libinput (xdisplay, device))
+    {
+        if (right_handed != -1)
+        {
+            GValue value = G_VALUE_INIT;
+
+            g_value_init (&value, G_TYPE_INT);
+            g_value_set_int (&value, !right_handed);
+
+            xfce_pointers_helper_change_property (device_info, device, xdisplay,
+                                                  LIBINPUT_PROP_LEFT_HANDED, &value);
+        }
+
+        if (reverse_scrolling != -1)
+        {
+            GValue value = G_VALUE_INIT;
+
+            g_value_init (&value, G_TYPE_INT);
+            g_value_set_int (&value, reverse_scrolling);
+
+            xfce_pointers_helper_change_property (device_info, device, xdisplay,
+                                                  LIBINPUT_PROP_NATURAL_SCROLL, &value);
+        }
+
+        return;
+    }
+#endif /* HAVE_LIBINPUT */
 
     /* search the number of buttons */
     for (n = 0, ptr = device_info->inputclassinfo; n < device_info->num_classes; n++)
@@ -361,7 +469,7 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
         }
 
         /* advance the offset */
-        ptr = (XAnyClassPtr) ((gchar *) ptr + ptr->length);
+        ptr = (XAnyClassPtr) (gpointer) ((gchar *) ptr + ptr->length);
     }
 
     if (num_buttons == 0)
@@ -373,9 +481,9 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
     /* allocate the button map */
     buttonmap = g_new0 (guchar, num_buttons);
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     XGetDeviceButtonMapping (xdisplay, device, buttonmap, num_buttons);
-    if (gdk_error_trap_pop () != 0)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0)
     {
         g_warning ("Failed to get button mapping");
         goto leave;
@@ -404,14 +512,21 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
                                                              5 /* scroll down */,
                                                              !reverse_scrolling))
             map_changed = TRUE;
+
+        if (xfce_pointers_helper_change_button_mapping_swap (buttonmap, num_buttons,
+                                                             6 /* scroll left */,
+                                                             7 /* scroll right */,
+                                                             !reverse_scrolling))
+
+            map_changed = TRUE;
     }
 
     /* only set on changes */
     if (map_changed)
     {
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         XSetDeviceButtonMapping (xdisplay, device, buttonmap, num_buttons);
-        if (gdk_error_trap_pop () != 0)
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0)
             g_warning ("Failed to set button mapping");
 
         /* don't put a hard time on ourselves and make debugging a lot better */
@@ -428,7 +543,7 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
                         device_info->name);
     }
 
-    leave:
+leave:
 
     g_free (buttonmap);
 }
@@ -447,23 +562,38 @@ xfce_pointers_helper_gcd (gint num,
 
 static void
 xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
-                                      XDevice     *device,
-                                      Display     *xdisplay,
-                                      gint         threshold,
-                                      gdouble      acceleration)
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      gint threshold,
+                                      gdouble acceleration)
 {
-    XFeedbackState      *states, *pt;
-    gint                 num_feedbacks;
-    XPtrFeedbackControl  feedback;
-    gint                 n;
-    gulong               mask = 0;
-    gint                 num, denom, gcd;
-    gboolean             found = FALSE;
+    XFeedbackState *states, *pt;
+    gint num_feedbacks;
+    XPtrFeedbackControl feedback;
+    gint n;
+    gulong mask = 0;
+    gint num, denom, gcd;
+    gboolean found = FALSE;
 
+#ifdef HAVE_LIBINPUT
+    if (xfce_pointers_is_libinput (xdisplay, device))
+    {
+        gdouble libinput_accel;
+        GValue value = G_VALUE_INIT;
+
+        libinput_accel = CLAMP ((acceleration / 5) - 1.0, -1.0, 1.0);
+        g_value_init (&value, G_TYPE_DOUBLE);
+        g_value_set_double (&value, libinput_accel);
+
+        xfce_pointers_helper_change_property (device_info, device, xdisplay,
+                                              LIBINPUT_PROP_ACCEL, &value);
+        return;
+    }
+#endif /* HAVE_LIBINPUT */
     /* get the feedback states for this device */
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     states = XGetFeedbackControl (xdisplay, device, &num_feedbacks);
-    if (gdk_error_trap_pop() != 0 || states == NULL)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || states == NULL)
     {
         g_critical ("Failed to get the feedback states of device %s",
                     device_info->name);
@@ -476,7 +606,7 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
         if (pt->class != PtrFeedbackClass)
         {
             /* advance the offset */
-            pt = (XFeedbackState *) ((gchar *) pt + pt->length);
+            pt = (XFeedbackState *) (gpointer) ((gchar *) pt + pt->length);
             continue;
         }
 
@@ -493,9 +623,9 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
 
         /* above 0 is a valid value, -1 is reset, -2.00
          * is passed if no change is required */
-        if (acceleration > 0 || acceleration == -1)
+        if (acceleration >= 0 || acceleration == -1)
         {
-            if (acceleration > 0)
+            if (acceleration >= 0)
             {
                 /* calculate the faction of the acceleration */
                 num = acceleration * MAX_DENOMINATOR;
@@ -523,10 +653,10 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
         }
 
         /* update the feedback of the device */
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         XChangeFeedbackControl (xdisplay, device, mask,
                                 (XFeedbackControl *) &feedback);
-        if (gdk_error_trap_pop() != 0)
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0)
         {
             g_warning ("Failed to set feedback states for device %s",
                        device_info->name);
@@ -553,10 +683,10 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
 
 
 static void
-xfce_pointers_helper_change_mode (XDeviceInfo  *device_info,
-                                  XDevice      *device,
-                                  Display      *xdisplay,
-                                  const gchar  *mode_name)
+xfce_pointers_helper_change_mode (XDeviceInfo *device_info,
+                                  XDevice *device,
+                                  Display *xdisplay,
+                                  const gchar *mode_name)
 {
     gint mode;
 
@@ -570,9 +700,9 @@ xfce_pointers_helper_change_mode (XDeviceInfo  *device_info,
         return;
     }
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     XSetDeviceMode (xdisplay, device, mode);
-    if (gdk_error_trap_pop () != 0)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0)
         g_critical ("Failed to change the device mode");
 
     xfsettings_dbg (XFSD_DEBUG_POINTERS,
@@ -584,7 +714,7 @@ xfce_pointers_helper_change_mode (XDeviceInfo  *device_info,
 static gchar *
 xfce_pointers_helper_device_xfconf_name (const gchar *name)
 {
-    GString     *string;
+    GString *string;
     const gchar *p;
 
     /* NOTE: this function exists in both the dialog and
@@ -615,31 +745,35 @@ xfce_pointers_helper_device_xfconf_name (const gchar *name)
 
 
 
-#ifdef DEVICE_PROPERTIES
+#if defined(DEVICE_PROPERTIES) || defined(HAVE_LIBINPUT)
 static void
-xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
-                                      XDevice      *device,
-                                      Display      *xdisplay,
-                                      const gchar  *prop_name,
+xfce_pointers_helper_change_property (XDeviceInfo *device_info,
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      const gchar *prop_name,
                                       const GValue *value)
 {
-    Atom         *props;
-    gint          n, n_props;
-    Atom          prop;
-    gchar        *atom_name;
-    Atom          type;
-    gint          format;
-    gulong        n_items, bytes_after, i;
-    gulong        n_succeeds;
-    Atom          float_atom;
-    GPtrArray    *array = NULL;
+    Atom *props;
+    gint n, n_props;
+    Atom prop;
+    gchar *atom_name;
+    Atom type;
+    gint format;
+    gulong n_items, bytes_after, i;
+    gulong n_succeeds;
+    Atom float_atom;
+    GPtrArray *array = NULL;
+    int rc;
     const GValue *val;
-    union {
+    union
+    {
         guchar *c;
         gshort *s;
-        glong  *l;
-        Atom   *a;
+        glong *l;
+        float *f;
+        Atom *a;
     } data;
+    guchar *allocated_data = NULL;
 
     /* assuming the device property never contained underscores... */
     atom_name = g_strdup (prop_name);
@@ -652,9 +786,20 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
     if (prop == None)
         return;
 
-    gdk_error_trap_push ();
+#ifdef HAVE_LIBINPUT
+    /*
+     * libinput cannot change properties on disabled devices
+     * see: https://bugs.freedesktop.org/show_bug.cgi?id=89296
+     * and: http://lists.x.org/archives/xorg-devel/2015-February/045716.html
+     */
+    if (prop != XInternAtom (xdisplay, DEVICE_ENABLED, True)
+        && !xfce_pointers_is_enabled (xdisplay, device))
+        return;
+#endif /* HAVE_LIBINPUT */
+
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     props = XListDeviceProperties (xdisplay, device, &n_props);
-    if (gdk_error_trap_pop () != 0 || props == NULL)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) || props == NULL)
         return;
 
     float_atom = XInternAtom (xdisplay, "FLOAT", False);
@@ -665,9 +810,11 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
         if (props[n] != prop)
             continue;
 
-        if (XGetDeviceProperty (xdisplay, device, prop, 0, 1000, False,
-                                AnyPropertyType, &type, &format,
-                                &n_items, &bytes_after, &data.c) == Success)
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
+        rc = XGetDeviceProperty (xdisplay, device, prop, 0, 1000, False,
+                                 AnyPropertyType, &type, &format,
+                                 &n_items, &bytes_after, &data.c);
+        if (!gdk_x11_display_error_trap_pop (gdk_display_get_default ()) && rc == Success)
         {
             if (n_items == 1
                 && (G_VALUE_HOLDS_INT (value)
@@ -675,17 +822,31 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
                     || G_VALUE_HOLDS_DOUBLE (value)))
             {
                 /* only 1 items to set */
-                val = value;
             }
-            else if (G_VALUE_TYPE (value) == XFCONF_TYPE_G_VALUE_ARRAY)
+            else if (G_VALUE_TYPE (value) == G_TYPE_PTR_ARRAY)
             {
                 array = g_value_get_boxed (value);
-                if (array->len != n_items)
+                /* LibInput array properties can have dynamic number of items.
+                   Do not enforce equal lengths, set as many items as defined in config
+                   and re-allocate the data array, if it is too small. */
+                if (array->len > n_items)
                 {
-                    g_critical ("Nr device property items (%ld) and xfconf value (%d) differ",
-                                n_items, array->len);
-                    break;
+                    switch (format)
+                    {
+                        case 8: allocated_data = (guchar *) g_new (guchar, array->len); break;
+                        case 16: allocated_data = (guchar *) g_new (gushort, array->len); break;
+                        case 32: allocated_data = (guchar *) g_new (gulong, array->len); break;
+                        default: allocated_data = NULL;
+                    }
+                    XFree (data.c);
+                    data.c = allocated_data;
+                    if (!allocated_data)
+                    {
+                        g_critical ("Unknown format %d for integer", format);
+                        break;
+                    }
                 }
+                n_items = array->len;
             }
             else
             {
@@ -730,7 +891,9 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
                          && type == float_atom
                          && format == 32)
                 {
-                    data.l[i] = g_value_get_double (val);
+                    /* Xorg actually uses sizeof(long) bytes per element if format == 32 */
+                    /* See https://gitlab.freedesktop.org/xorg/app/xinput/-/blob/cef07c0c8280d7e7b82c3bcc62a1dfbe8cc43ff8/src/property.c#L80 */
+                    *(float *) &data.l[i] = (float) g_value_get_double (val);
                 }
                 else
                 {
@@ -745,10 +908,11 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
 
             if (n_succeeds == n_items)
             {
-                gdk_error_trap_push ();
+                gdk_x11_display_error_trap_push (gdk_display_get_default ());
                 XChangeDeviceProperty (xdisplay, device, prop, type, format,
                                        PropModeReplace, data.c, n_items);
-                if (gdk_error_trap_pop () != 0)
+                XSync (xdisplay, FALSE);
+                if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()))
                 {
                     g_critical ("Failed to set device property %s for %s",
                                 prop_name, device_info->name);
@@ -758,25 +922,29 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
                                 "[%s] Changed device property %s",
                                 device_info->name, prop_name);
             }
-
-            XFree (data.c);
         }
 
+        if (allocated_data)
+            g_free (allocated_data);
+        else if (data.c)
+            XFree (data.c);
+
         break;
     }
 
     XFree (props);
 }
+#endif /* DEVICE_PROPERTIES || HAVE_LIBINPUT */
 
 
-
+#ifdef DEVICE_PROPERTIES
 static void
 xfce_pointers_helper_change_properties (gpointer key,
                                         gpointer value,
                                         gpointer user_data)
 {
     XfcePointerData *pointer_data = user_data;
-    const gchar     *prop_name = ((gchar *) key) + pointer_data->prop_name_len;
+    const gchar *prop_name = ((gchar *) key) + pointer_data->prop_name_len;
 
     xfce_pointers_helper_change_property (pointer_data->device_info,
                                           pointer_data->device,
@@ -789,27 +957,26 @@ xfce_pointers_helper_change_properties (gpointer key,
 
 static void
 xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
-                                      XID                *xid)
+                                      XID *xid)
 {
-    Display         *xdisplay = GDK_DISPLAY ();
-    XDeviceInfo     *device_list, *device_info;
-    gint             n, ndevices;
-    XDevice         *device;
-    gchar           *device_name;
-    gchar            prop[256];
-    gboolean         right_handed;
-    gboolean         reverse_scrolling;
-    gint             threshold;
-    gdouble          acceleration;
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    XDeviceInfo *device_list, *device_info;
+    gint n, ndevices;
+    XDevice *device;
+    gchar *device_name;
+    gchar prop[256];
+    gboolean right_handed;
+    gboolean reverse_scrolling;
+    gint threshold;
+    gdouble acceleration;
 #ifdef DEVICE_PROPERTIES
-    GHashTable      *props;
-    XfcePointerData  pointer_data;
+    GHashTable *props;
+    XfcePointerData pointer_data;
 #endif
-    const gchar     *mode;
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (gdk_display_get_default ());
     device_list = XListInputDevices (xdisplay, &ndevices);
-    if (gdk_error_trap_pop () != 0 || device_list == NULL)
+    if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device_list == NULL)
     {
         g_message ("No input devices found");
         return;
@@ -817,6 +984,8 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
 
     for (n = 0; n < ndevices; n++)
     {
+        gchar *mode;
+
         /* filter the pointer devices */
         device_info = &device_list[n];
         if (device_info->use != IsXExtensionPointer
@@ -828,9 +997,9 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
             continue;
 
         /* open the device */
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         device = XOpenDevice (xdisplay, device_info->id);
-        if (gdk_error_trap_pop () != 0 || device == NULL)
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device == NULL)
         {
             g_critical ("Unable to open device %s", device_info->name);
             continue;
@@ -867,10 +1036,14 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
 
         /* read mode settings */
         g_snprintf (prop, sizeof (prop), "/%s/Mode", device_name);
-        mode =  xfconf_channel_get_string  (helper->channel, prop, NULL);
+        mode = xfconf_channel_get_string (helper->channel, prop, NULL);
 
         if (mode != NULL)
+        {
             xfce_pointers_helper_change_mode (device_info, device, xdisplay, mode);
+            g_free (mode);
+            mode = NULL;
+        }
 
 #ifdef DEVICE_PROPERTIES
         /* set device properties */
@@ -900,23 +1073,24 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
 
 
 static void
-xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
-                                               const gchar        *property_name,
-                                               const GValue       *value,
+xfce_pointers_helper_channel_property_changed (XfconfChannel *channel,
+                                               const gchar *property_name,
+                                               const GValue *value,
                                                XfcePointersHelper *helper)
 {
-    Display      *xdisplay = GDK_DISPLAY ();
-    XDeviceInfo  *device_list, *device_info;
-    XDevice      *device;
-    gint          n, ndevices;
-    gchar       **names;
-    gchar        *device_name;
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    XDeviceInfo *device_list, *device_info;
+    XDevice *device;
+    gint n, ndevices;
+    gchar **names;
+    gchar *device_name;
 
     if (G_UNLIKELY (property_name == NULL))
-         return;
+        return;
 
     /* check the daemon status */
-    if (strcmp (property_name, "/DisableTouchpadWhileTyping") == 0)
+    if (strcmp (property_name, "/DisableTouchpadWhileTyping") == 0
+        || strcmp (property_name, "/DisableTouchpadDuration") == 0)
     {
         xfce_pointers_helper_syndaemon_check (helper);
         return;
@@ -927,9 +1101,9 @@ xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
 
     if (names != NULL && g_strv_length (names) >= 2)
     {
-        gdk_error_trap_push ();
+        gdk_x11_display_error_trap_push (gdk_display_get_default ());
         device_list = XListInputDevices (xdisplay, &ndevices);
-        if (gdk_error_trap_pop () != 0 || device_list == NULL)
+        if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device_list == NULL)
         {
             g_message ("No input devices found");
             return;
@@ -948,9 +1122,9 @@ xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
             if (strcmp (names[0], device_name) == 0)
             {
                 /* open the device */
-                gdk_error_trap_push ();
+                gdk_x11_display_error_trap_push (gdk_display_get_default ());
                 device = XOpenDevice (xdisplay, device_info->id);
-                if (gdk_error_trap_pop () != 0 || device == NULL)
+                if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()) != 0 || device == NULL)
                 {
                     g_critical ("Unable to open device %s", device_info->name);
                     continue;
@@ -1015,12 +1189,12 @@ xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
 #ifdef DEVICE_HOTPLUGGING
 static GdkFilterReturn
 xfce_pointers_helper_event_filter (GdkXEvent *xevent,
-                                   GdkEvent  *gdk_event,
-                                   gpointer   user_data)
+                                   GdkEvent *gdk_event,
+                                   gpointer user_data)
 {
-    XEvent                     *event = xevent;
+    XEvent *event = xevent;
     XDevicePresenceNotifyEvent *dpn_event = xevent;
-    XfcePointersHelper         *helper = XFCE_POINTERS_HELPER (user_data);
+    XfcePointersHelper *helper = XFCE_POINTERS_HELPER (user_data);
 
     if (event->type == helper->device_presence_event_type)
     {
